;
; *.*
;   OCM-EXTRA Pack / Switched I/O ports ($40-$4F) Revision 1 or later
;
; Copyright (c) 2011-2023 KdL
; All rights reserved.
;
; Redistribution and use of this source code or any derivative works, are
; permitted provided that the following conditions are met:
;
; 1. Redistributions of source code must retain the above copyright notice,
;    this list of conditions and the following disclaimer.
; 2. Redistributions in binary form must reproduce the above copyright
;    notice, this list of conditions and the following disclaimer in the
;    documentation and/or other materials provided with the distribution.
; 3. Redistributions may not be sold, nor may they be used in a commercial
;    product or activity without specific prior written permission.
;
; THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
; "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
; TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
; PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
; CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
; EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
; PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
; WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
; OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
; ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;
; ----------------------------------------
;     Prog: OCM-CPUSPEED v1.4 by KdL
;     Date: 2023.10.11
;     Note: OCM-PLD Pack v2.4 or later
;     Coded in TASM80 v3.2ud w/ TWZ'CA3
;     TASM is at http://www.ticalc.org
; ----------------------------------------
;

            .ORG  $0100

FN1         .EQU  'c'                     ; strings and options
FN2         .EQU  'p'
FN3         .EQU  'u'
FN4         .EQU  's'
FN5         .EQU  'p'
FN6         .EQU  'e'
FN7         .EQU  'e'
FN8         .EQU  'd'
VER1        .EQU  '1'
VER2        .EQU  '4'
YR1         .EQU  '2'
YR2         .EQU  '0'
YR3         .EQU  '2'
YR4         .EQU  '3'
VERBOSE     .EQU  '/'
QUIET       .EQU  '-'
Z80A        .EQU  'a'
HYBA        .EQU  'b'
TPRA        .EQU  'c'
Z80B        .EQU  'd'
HYBB        .EQU  'e'
OPT1        .EQU  '0'
OPT2        .EQU  '1'
OPT3        .EQU  '2'
OPT4        .EQU  '3'
OPT5        .EQU  '4'
OPT6        .EQU  '5'
OPT7        .EQU  '6'
OPT8        .EQU  '7'
RESTORE     .EQU  'r'
LOCK        .EQU  'l'
UNLOCK      .EQU  'u'
HLP1        .EQU  'h'
HLP2        .EQU  '?'

REDIROFF    .EQU  $01
REDIRON     .EQU  REDIROFF+1
SCMDOPT1    .EQU  $03
SCMDBASE    .EQU  OPT1-SCMDOPT1
PANAID      .EQU  $08
HYBRIDOFF   .EQU  $0b
HYBRIDON    .EQU  HYBRIDOFF+1
SCMDLOCK    .EQU  $29
SCMDUNLK    .EQU  SCMDLOCK+1
SWIOADDR    .EQU  $40                     ; device id register of switched i/o ports
SCMDADDR    .EQU  $41                     ; smart command port
VDIPADDR    .EQU  $42                     ; virtual dip-sw port
LOCKADDR    .EQU  $43                     ; lock mask port
CUSTSTAT    .EQU  $47                     ; custom turbo port
PANASTAT    .EQU  $48
REVADDR     .EQU  $4f
NULID       .EQU  $80
OCMID       .EQU  $d4                     ; ocm id
SCMDREST    .EQU  SCMDBASE+OCMID+1-256

TURBOMASK   .EQU  %00000001               ; masks
CUSTMASK    .EQU  %00000111
TMSDMASK    .EQU  %00001000
REDIRMASK   .EQU  %00010000
MODE1       .EQU  %00000001               ; high-speed modes
MODE2       .EQU  MODE1+1
MODE3       .EQU  MODE2+1
MODE4       .EQU  MODE3+1
MODE5       .EQU  MODE4+1
MODE6       .EQU  MODE5+1
MODE7       .EQU  MODE6+1
TURBOLOCK   .EQU  %00000001
LOWERCASE   .EQU  %00100000
UP          .EQU  -LOWERCASE
_           .EQU  %10000000               ; obfuscation mask, e.g. 'A'|_,'B'|_
DEOBF       .EQU  %01111111               ; de-obfuscation mask
REVMASK     .EQU  %00011111

NUL         .EQU  $00                     ; ascii table
LF          .EQU  $0a
CLS         .EQU  $0c
CR          .EQU  $0d
EOF         .EQU  $1a
SPC         .EQU  $20

PLENGHT     .EQU  $0080                   ; system calls
CALSLT      .EQU  $001c
CHPUT       .EQU  $00a2
EXPTBL      .EQU  $fcc1
_BDOS       .EQU  $0005
_STROUT     .EQU  $09                     ; string output

UIOPT       .EQU  $08                     ; error code 8  : *** Invalid option
UIPARM      .EQU  $0b                     ; error code 11 : *** Invalid parameter
UUNCMD      .EQU  $0c                     ; error code 12 : *** Unsupported command
UUNEXP      .EQU  $1c                     ; error code 28 : *** Unexpected error
_TERM       .EQU  $62                     ; terminate with error code

ONE         .EQU  $01                     ;   1 byte  for char
HLN         .EQU  $10                     ;  16 bytes for hex line
CLU         .EQU  $0100                   ; 256 bytes for cluster

K1          .EQU  'K'                     ; editable copyleft: 1st three chars
K2          .EQU  'd'
K3          .EQU  'L'
_1          .EQU  K1|_                    ; obfuscated copyleft
_2          .EQU  K2|_
_3          .EQU  K3|_
; ----------------------------------------
startProg:
            jr    mainProg                ; type support, A:\>TYPE FILENAME.COM
typeMsg:
;           .DB   CR,SPC,SPC,SPC,SPC,LF   ; 'jr mainProg' ctrl sequences mask
            .DB   CR,"OCM-"
            .DB   FN1+UP,FN2+UP,FN3+UP,FN4+UP,FN5+UP,FN6+UP,FN7+UP,FN8+UP
            .DB   " v",VER1,".",VER2," by "
clearCopyleft:
            .DB   K1,K2,K3," 2011-",YR1,YR2,YR3,YR4,CR
msgLF:
            .DB   LF,EOF
; ----------------------------------------
EXTRA:      .DB   NUL                     ; vars
HYBRID:     .DB   HYBRIDON
REDIR:      .DB   REDIRON
EXITCODE:   .DB   NUL                     ; error code 0  : (no error)
; ----------------------------------------
mainProg:
            di
            ld    a,OCMID
            out   (SWIOADDR),a            ; set OCM ID
            ld    c,LF+ONE                ; init 'call strDisp' to use
;           ld    d,NULID+SCMDBASE        ; reg d is 'OPTION'
;           ld    e,VERBOSE               ; reg e is 'PREFIX'
            ld    de,(NULID+SCMDBASE)*256+VERBOSE
copyleftChk:
            ld    hl,clearCopyleft
            ld    a,_1
            and   DEOBF                   ; de-obfuscation
            cp    (hl)
            jr    nz,copyleftNext         ; protection: 'K' is confirmed here
            inc   hl
            ld    a,_2
            and   DEOBF                   ; de-obfuscation
            cp    (hl)
copyleftNext:
            jr    nz,jrnzSPC              ; protection: 'd' is confirmed here
            inc   hl
            ld    a,_3
            and   DEOBF                   ; de-obfuscation
            cp    (hl)
jrnzSPC:
            jr    nz,noParam              ; protection: parameter down!
; ----------------------------------------
startParser:
            ld    hl,PLENGHT
            ld    b,(hl)
            ld    a,b
            cp    NUL
            jr    z,noParam               ; jump if no chars
            ld    a,(PLENGHT+ONE)
            cp    SPC                     ; ignore this policy:
            jr    nz,noParam              ; e.g. A:\>FILENAME/x
preLoop:
            inc   hl
            ld    a,(jrnzSPC)             ; protection: jrnzSPC = $20
            cp    (hl)
            jr    nz,getParam             ; get 1st parameter:
            djnz  preLoop                 ; e.g. A:\>FILENAME      -x
noParam:
            jp    ocmidChk                ; no parameters
getParam:
            ld    a,(hl)
            ld    e,a
            djnz  getOption               ; if no option, get invalid parameter
invalidParam:
            ld    a,UIPARM
            ld    (EXITCODE),a            ; set error code 11
            ld    hl,errorMsg0
            call  strDisp                 ; 'invalid parameter' without prefix
            jr    nearestExit             ; jump to the nearest exit, 2 bytes
getOption:
            inc   hl
            ld    a,(hl)
            or    LOWERCASE               ; 1st option forced to lowercase
            ld    d,a
            djnz  nextOption              ; if end of PLENGHT, parameter is get
            jr    prefixChk
nextOption:
            inc   hl
            ld    a,(hl)
            or    LOWERCASE               ; 2nd option forced to lowercase
            cp    LOCK
            jr    z,set2ndOption          ; is 'l'
            cp    UNLOCK
            jr    nz,postLoop1            ; is other
set2ndOption:
            ld    (EXTRA),a               ; is 'u'
            jr    postLoop2
postLoop:
            inc   hl
postLoop1:
            ld    a,(jrnzSPC)             ; protection: jrnzSPC = $20
            cp    (hl)
            jr    nz,invalidParam         ; get 2nd parameter as invalid:
postLoop2:
            djnz  postLoop                ; e.g. A:\>FILENAME -x    abcdef
prefixChk:
            ld    a,e
            cp    QUIET                   ; '-'
            jr    z,paramChk
            cp    VERBOSE                 ; '/'
            jr    nz,invalidParam
; ----------------------------------------
paramChk:
            ld    a,d
            cp    Z80A
            jr    z,optAD                 ; jump if 'a'
            cp    Z80B
            jr    z,optAD                 ; jump if 'd'
            cp    HYBA
            jr    z,optBE                 ; jump if 'b'
            cp    HYBB
            jr    z,optBE                 ; jump if 'e'
            cp    TPRA
            jr    z,optC                  ; jump if 'c'
            cp    RESTORE
            jr    z,ocmidChk              ; jump if 'r'
            sub   OPT1-ONE
            ld    b,OPT8-OPT1+ONE
optLoop:
            cp    b
            jr    z,ocmidChk              ; jump if '7' to '0' (loop 8 to 1)
            djnz  optLoop
            ld    a,(EXTRA)
            cp    b
            jr    nz,invalidParam         ; invalid with '/l?', '-u?', etc.
            ld    a,d
            cp    LOCK
            jr    z,extraOnly             ; jump if 'l'
            cp    UNLOCK
            jr    z,extraOnly             ; jump if 'u'
            cp    HLP1
            jr    z,isHelp                ; jump if 'h'
            cp    HLP2
            jr    nz,invalidOption        ; jump if '?'
isHelp:
            ld    hl,helpMsg              ; help message
nearestExit:
            jp    lastDisp
invalidOption:
            ld    a,UIOPT
            ld    (EXITCODE),a            ; set error code 8
            ld    hl,errorMsg0
            call  strDisp
            ld    hl,errorMsg2
            jr    nearestExit             ; 'invalid option'
optC:
            ld    d,OPT1
optAD:
            ld    a,HYBRIDOFF
            ld    (HYBRID),a
            jr    ocmidChk
optBE:
            dec   d
            ld    a,HYBRIDON
            ld    (HYBRID),a
            jr    ocmidChk
setPana:
            ld    a,REDIROFF
            out   (SCMDADDR),a            ; set redirection off
            ld    a,PANAID
            out   (SWIOADDR),a            ; set PANA ID
            ret
setZ80A:
            call  setPana                 ; out $40,8 = PANA specification
            ld    a,ONE                   ; out $41,1 = z80a 3.58MHz
            jr    exitZ80B
setZ80B:
            call  setPana                 ; out $40,8 = PANA specification
            ld    a,NUL                   ; out $41,0 = z80b 5.37MHz
exitZ80B:
            out   (SCMDADDR),a
            ld    a,OCMID
            out   (SWIOADDR),a            ; set OCM ID
            jr    extraChk
extraOnly:
            ld    (EXTRA),a               ; shift the option to extra
            ld    d,NULID+SCMDBASE        ; $80 is the null smart command
; ----------------------------------------
ocmidChk:
            in    a,(SWIOADDR)            ; check for OCM ID
            cpl
            cp    OCMID
; ----------------------------------------
            jp    nz,isUnsupp             ; 'unsupported command' (rem to test)
            in    a,(REVADDR)             ; revision check
            and   REVMASK
            cp    NUL                     ; = revision 0
            jp    z,isUnsupp              ; 'unsupported command' (jr nz to test)
; ----------------------------------------
            ld    a,d
            cp    NULID+SCMDBASE
            jr    z,extraChk
            cp    RESTORE
            jr    z,outRestore
            ld    a,(HYBRID)
            out   (SCMDADDR),a            ; set hybrid clock
            ld    a,d
            cp    Z80A
            jr    z,setZ80A
            cp    Z80B
            jr    z,setZ80B
            ld    a,(REDIR)
            out   (SCMDADDR),a            ; set redirection
            ld    a,d
            jr    outScmd
outRestore:
            ld    a,SCMDREST
outScmd:
            sub   SCMDBASE
            out   (SCMDADDR),a            ; set smart command
extraChk:
            ld    a,(EXTRA)
            cp    UNLOCK
            jr    nz,lockChk
            ld    a,SCMDUNLK
            jr    setExtra
lockChk:
            cp    LOCK
            jr    nz,unexpChk
            ld    a,SCMDLOCK
setExtra:
            out   (SCMDADDR),a            ; lock toggle
unexpChk:
            in    a,(CUSTSTAT)
            and   CUSTMASK
            cp    NUL
            jr    nz,quietChk
            ld    a,UUNEXP
            ld    (EXITCODE),a            ; set error code 28
            ld    hl,unexpMsg             ; 'unexpected error' if CUSTSTAT is 'xxxxx000'
            jp    lastDisp
quietChk:
            ld    a,e
            cp    QUIET
            jp    z,mainExit              ; quiet exit (useful for batch)
; ----------------------------------------
            ld    hl,msgT80               ; 'T80 @ '
            call  strDisp
getStatus:
            in    a,(PANASTAT)
            and   ONE
            cp    NUL
            jr    z,getOthers
            ld    hl,msg537               ; 5.37MHz
            jr    isMHz
getOthers:
            in    a,(VDIPADDR)            ; get many states
            and   TURBOMASK               ; and filter for turbo
            cp    NUL
            jr    nz,isMode1              ; MODE0: VDIPADDR is 'xx000xxx'
is358:
            ld    hl,msg358               ; 3.58MHz
            jr    isMHz
isMode1:
            in    a,(CUSTSTAT)
            and   CUSTMASK
            cp    MODE1
            jr    nz,isMode2              ; MODE1: CUSTSTAT is 'xxxxx001'
            ld    hl,msg410
            jr    isMhz
isMode2:
            cp    MODE2                   ; MODE2: CUSTSTAT is 'xxxxx010'
            jr    nz,isMode3
            ld    hl,msg448
            jr    isMHz
isMode3:
            cp    MODE3                   ; MODE3: CUSTSTAT is 'xxxxx011'
            jr    nz,isMode4
            ld    hl,msg490
            jr    isMHz
isUnsupp:
            ld    a,UUNCMD
            ld    (EXITCODE),a            ; set error code 12
            ld    hl,unsuppMsg            ; 'unsupported command'
            jr    lastDisp
isMode4:
            cp    MODE4                   ; MODE4: CUSTSTAT is 'xxxxx100'
            jr    nz,isMode5
            ld    hl,msg539
            jr    isMHz
isMode5:
            cp    MODE5                   ; MODE5: CUSTSTAT is 'xxxxx101'
            jr    nz,isMode6
            ld    hl,msg610
            jr    isMHz
isMode6:
            cp    MODE6                   ; MODE6: CUSTSTAT is 'xxxxx110'
            jr    nz,isMode7
            ld    hl,msg696
            jr    isMHz
isMode7:
            ld    hl,msg806               ; MODE7: CUSTSTAT is 'xxxxx111'
isMHz:
            call  strDisp
            ld    hl,msgMHz
            call  strDisp
            in    a,(CUSTSTAT)            ; tMSD (=tMegaSD) check
            and   TMSDMASK
            cp    NUL
            jr    z,nextChk
            ld    hl,tMegaSD
            call  strDisp
nextChk:
            in    a,(CUSTSTAT)            ; tPR (=tPanaRedir) check
            and   REDIRMASK
            cp    NUL
            jr    z,isLock
            ld    hl,tPanaRedir
            call  strDisp
isLock:
            ld    b,TURBOLOCK             ; locked/unlocked message
            call  strLock
; ----------------------------------------
lastDisp:
            dec   c                       ; use 'jr lastDisp' or 'jp lastDisp'
strDisp:
            inc   c                       ; use 'call strDisp'
charLoop:
            ld    a,(hl)                  ; load 1st char, ascii < 127 only
            and   DEOBF                   ; de-obfuscation
            ld    ix,CHPUT                ; init chput
            ld    iy,(EXPTBL-1)           ; init main-rom slot
            call  CALSLT                  ; NUL and EOF are not valid as 1st
            inc   hl
            xor   a
            cp    (hl)
            jr    nz,nextChar             ; NUL,n = SPC(n)
            inc   hl
            ld    b,(hl)                  ; b = n
            ld    a,(jrnzSPC)             ; protection: jrnzSPC = $20
;           and   DEOBF                   ; de-obfuscation (optional)
spcLoop:
            ld    ix,CHPUT                ; init chput
            ld    iy,(EXPTBL-1)           ; init main-rom slot
            call  CALSLT                  ; print SPC until b = 0
            djnz  spcLoop
            inc   hl
nextChar:
            ld    a,EOF
            cp    (hl)
            jr    nz,charLoop             ; EOF = end of string
            inc   hl                      ; point to the next string at exit
            dec   c
            ld    a,LF
            cp    c                       ; c = LF is 'lastDisp'
            ret   nz                      ; exit of 'strDisp'
            ld    de,strLFCR              ; print LF + CR after 'lastDisp'
            ld    c,_STROUT               ; 'OPTION' and 'PREFIX' will be lost
            call  _BDOS
mainExit:
            xor   a
            out   (SWIOADDR),a            ; cancel Device ID
            ld    (PLENGHT),a             ; reset PLENGHT for the next command
            ei
            ld    a,(EXITCODE)
;           cp    NUL                     ; allow 'call lastDisp'
;           ret   z                       ; if error code = 0 (optional)
            ld    b,a
            ld    c,_TERM                 ; terminate with error code (MSX-DOS 2 only)
            jp    _BDOS                   ; if MSX-DOS 1, it acts like 'ret'
strLFCR:
            .DB   LF,CR,'$'
; ----------------------------------------
strLock:
            in    a,(LOCKADDR)
            and   b                       ; b is required as lock mask here
            cp    NUL
            jr    z,msgUnlock
            ld    hl,lockMsg
            ret
msgUnlock:
            ld    hl,unlockMsg
            ret
; ----------------------------------------
helpMsg:                                  ; texts comply with 37 columns
            .DB   LF|_
            .DB   CLS|_,'S'|_,'y'|_,'n'|_,'t'|_,'a'|_,'x'|_,':'|_,' '|_
            .DB   FN1|_,FN2|_,FN3|_,FN4|_,FN5|_,FN6|_,FN7|_,FN8|_,' '|_
            .DB   '['|_,'{'|_,VERBOSE|_,'|'|_,QUIET|_,'}'|_
            .DB   'o'|_,'p'|_,'t'|_,'i'|_,'o'|_,'n'|_
            .DB   '['|_,'l'|_,'|'|_,'u'|_,']'|_,']'|_
            .DB   LF|_

            .DB   LF|_,CR|_,'N'|_,'o'|_,' '|_
            .DB   'p'|_,'a'|_,'r'|_,'a'|_,'m'|_,'e'|_,'t'|_,'e'|_,'r'|_,':'|_
            .DB   NUL,4,'S'|_,'h'|_,'o'|_,'w'|_,'s'|_,' '|_,'t'|_,'h'|_,'e'|_
            .DB   ' '|_,'s'|_,'t'|_,'a'|_,'t'|_,'u'|_,'s'|_
            .DB   LF|_

            .DB   LF|_,CR|_
            .DB   'P'|_,'r'|_,'e'|_,'f'|_,'i'|_,'x'|_,'e'|_,'s'|_,':'|_
            .DB   NUL,4,VERBOSE|_,NUL,3
            .DB   'V'|_,'e'|_,'r'|_,'b'|_,'o'|_,'s'|_,'e'|_,' '|_
            .DB   'o'|_,'u'|_,'t'|_,'p'|_,'u'|_,'t'|_
            .DB   LF|_,CR|_,NUL,13,QUIET|_,NUL,3
            .DB   'Q'|_,'u'|_,'i'|_,'e'|_,'t'|_,' '|_
            .DB   'o'|_,'u'|_,'t'|_,'p'|_,'u'|_,'t'|_
            .DB   LF|_

            .DB   LF|_,CR|_,'O'|_,'p'|_,'t'|_,'i'|_,'o'|_,'n'|_,'s'|_,':'|_
            .DB   NUL,5,Z80A|_,NUL,3,'3'|_,'.'|_,'5'|_,'8'|_,'M'|_,'H'|_,'z'|_
            .DB   LF|_,CR|_,NUL,13,HYBA|_,NUL,3,'3'|_,'.'|_,'5'|_,'8'|_
            .DB   'M'|_,'H'|_,'z'|_,' '|_,'t'|_,'M'|_,'S'|_,'D'|_
            .DB   LF|_,CR|_,NUL,13,TPRA|_,NUL,3,'3'|_,'.'|_,'5'|_,'8'|_
            .DB   'M'|_,'H'|_,'z'|_,' '|_,'t'|_,'P'|_,'R'|_
            .DB   LF|_,CR|_,NUL,13,Z80B|_
            .DB   NUL,3,'5'|_,'.'|_,'3'|_,'7'|_,'M'|_,'H'|_,'z'|_
            .DB   LF|_,CR|_,NUL,13,HYBB|_,NUL,3,'5'|_,'.'|_,'3'|_,'7'|_
            .DB   'M'|_,'H'|_,'z'|_,' '|_,'t'|_,'M'|_,'S'|_,'D'|_
            .DB   LF|_,CR|_,NUL,13,OPT1|_
            .DB   NUL,3,'3'|_,'.'|_,'5'|_,'8'|_,'M'|_,'H'|_,'z'|_,' '|_
            .DB   't'|_,'M'|_,'S'|_,'D'|_,' '|_,'t'|_,'P'|_,'R'|_
            .DB   LF|_,CR|_,NUL,13,'|'|_,NUL,6,'|'|_,NUL,6,'|'|_,NUL,3,'|'|_
            .DB   LF|_,CR|_,NUL,13,OPT8|_
            .DB   NUL,3,'8'|_,'.'|_,'0'|_,'6'|_,'M'|_,'H'|_,'z'|_,' '|_
            .DB   't'|_,'M'|_,'S'|_,'D'|_,' '|_,'t'|_,'P'|_,'R'|_
            .DB   LF|_,CR|_,NUL,13,RESTORE|_
            .DB   NUL,3,'R'|_,'e'|_,'s'|_,'t'|_,'o'|_,'r'|_,'e'|_,' '|_
            .DB   'd'|_,'e'|_,'f'|_,'a'|_,'u'|_,'l'|_,'t'|_
            .DB   LF|_

            .DB   LF|_,CR|_,'E'|_,'x'|_,'t'|_,'r'|_,'a'|_,'s'|_,':'|_,NUL,6
            .DB   LOCK|_,NUL,3,'T'|_,'o'|_,'g'|_,'g'|_,'l'|_,'e'|_,'s'|_,' '|_
            .DB   'l'|_,'o'|_,'c'|_,'k'|_,'e'|_,'d'|_,LF|_,CR|_,NUL,13
            .DB   UNLOCK|_,NUL,3,'T'|_,'o'|_,'g'|_,'g'|_,'l'|_,'e'|_,'s'|_,' '|_
            .DB   'u'|_,'n'|_,'l'|_,'o'|_,'c'|_,'k'|_,'e'|_,'d'|_
            .DB   LF|_

            .DB   LF|_,CR|_
            .DB   'E'|_,'x'|_,'a'|_,'m'|_,'p'|_,'l'|_,'e'|_,'s'|_,':'|_
            .DB   NUL,8,FN1|_,FN2|_,FN3|_,FN4|_,FN5|_,FN6|_,FN7|_,FN8|_,' '|_
            .DB   VERBOSE|_,UNLOCK|_,LF|_,CR|_
            .DB   NUL,17,FN1|_,FN2|_,FN3|_,FN4|_,FN5|_,FN6|_,FN7|_,FN8|_,' '|_
            .DB   QUIET|_,OPT5|_,LOCK|_

            .DB   EOF
msgT80:
            .DB   'T'|_,'8'|_,'0'|_,' '|_,'@'|_,' '|_,EOF
msg358:
            .DB   '3'|_,'.'|_,'5'|_,'8'|_,EOF
msg537:
            .DB   '5'|_,'.'|_,'3'|_,'7'|_,EOF
msg410:
            .DB   '4'|_,'.'|_,'1'|_,'0'|_,EOF
msg448:
            .DB   '4'|_,'.'|_,'4'|_,'8'|_,EOF
msg490:
            .DB   '4'|_,'.'|_,'9'|_,'0'|_,EOF
msg539:
            .DB   '5'|_,'.'|_,'3'|_,'9'|_,EOF
msg610:
            .DB   '6'|_,'.'|_,'1'|_,'0'|_,EOF
msg696:
            .DB   '6'|_,'.'|_,'9'|_,'6'|_,EOF
msg806:
            .DB   '8'|_,'.'|_,'0'|_,'6'|_,EOF
msgMHz:
            .DB   'M'|_,'H'|_,'z'|_,' '|_,EOF
tMegaSD:
            .DB   '('|_,'t'|_,'M'|_,'S'|_,'D'|_,')'|_,' '|_,EOF
tPanaRedir:
            .DB   '('|_,'t'|_,'P'|_,'R'|_,')'|_,' '|_,EOF
unlockMsg:
            .DB   'u'|_,'n'|_
lockMsg:
            .DB   'l'|_,'o'|_,'c'|_,'k'|_,'e'|_,'d'|_,EOF
errorMsg0:
            .DB   LF|_,'*'|_,'*'|_,'*'|_,' '|_
            .DB   'I'|_,'n'|_,'v'|_,'a'|_,'l'|_,'i'|_,'d'|_,' '|_,EOF
errorMsg1:
            .DB   'p'|_,'a'|_,'r'|_,'a'|_,'m'|_,'e'|_,'t'|_,'e'|_,'r'|_,EOF
errorMsg2:
            .DB   'o'|_,'p'|_,'t'|_,'i'|_,'o'|_,'n'|_,EOF
unsuppMsg:
            .DB   LF|_,'*'|_,'*'|_,'*'|_,' '|_
            .DB   'U'|_,'n'|_,'s'|_,'u'|_,'p'|_,'p'|_,'o'|_,'r'|_,'t'|_,'e'|_
            .DB   'd'|_,' '|_,'c'|_,'o'|_,'m'|_,'m'|_,'a'|_,'n'|_,'d'|_,EOF
unexpMsg:
            .DB   LF|_,'*'|_,'*'|_,'*'|_,' '|_
            .DB   'U'|_,'n'|_,'e'|_,'x'|_,'p'|_,'e'|_,'c'|_,'t'|_,'e'|_,'d'|_
            .DB   ' '|_,'e'|_,'r'|_,'r'|_,'o'|_,'r'|_,EOF
; ----------------------------------------
startFill:                                ; $FF fill the 1st hex line
;           .FILL ((((startFill-startProg)/HLN)+ONE)*HLN-(startFill-startProg))
endProg:                                  ; $FF fill the cluster
;           .FILL ((((endProg-startProg)/CLU)+ONE)*CLU-(endProg-startProg))-HLN
                                          ; [ FILENAME.COM ] at last hex line
            .DB   "[ ",FN1+UP,FN2+UP,FN3+UP,FN4+UP,FN5+UP,FN6+UP,FN7+UP,FN8+UP
            .DB   ".COM ]"

            .END
