;
; *.*
;   OCM-EXTRA Pack
;
; Copyright (c) 2019-2022 KdL
; All rights reserved.
;
; Redistribution and use of this source code or any derivative works, are
; permitted provided that the following conditions are met:
;
; 1. Redistributions of source code must retain the above copyright notice,
;    this list of conditions and the following disclaimer.
; 2. Redistributions in binary form must reproduce the above copyright
;    notice, this list of conditions and the following disclaimer in the
;    documentation and/or other materials provided with the distribution.
; 3. Redistributions may not be sold, nor may they be used in a commercial
;    product or activity without specific prior written permission.
;
; THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
; "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
; TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
; PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
; CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
; EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
; PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
; WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
; OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
; ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;
; ----------------------------------------
;     Prog: SMX-PLDFLASH v1.2 by KdL
;     Date: 2022.09.24
;     Coded in TASM80 v3.2ud w/ TWZ'CA3
;     TASM is at http://www.ticalc.org
; ----------------------------------------
;     Based on PLDLOAD by NAKAYAMA Atsushi
;     LICENSE-2.0 at the end of this file
; ----------------------------------------
;     Target device: EPCS16 or higher
;     Excluded: EPCS1 and EPCS4
; ----------------------------------------
;

              .ORG  $0100

FN1           .EQU  's'                   ; strings and options
FN2           .EQU  'm'
FN3           .EQU  'x'
FN4           .EQU  'f'
FN5           .EQU  'l'
FN6           .EQU  'a'
FN7           .EQU  's'
FN8           .EQU  'h'
VER1          .EQU  '1'
VER2          .EQU  '2'
YR1           .EQU  '2'
YR2           .EQU  '0'
YR3           .EQU  '2'
YR4           .EQU  '2'

SWIOADDR      .EQU  $40                   ; device id register of switched i/o ports
SCMDADDR      .EQU  $41                   ; smart command port
OCMID         .EQU  $d4                   ; ocm id

LOWERCASE     .EQU  %00100000             ; masks
UP            .EQU  -LOWERCASE

LF            .EQU  $0a                   ; ascii table
CLS           .EQU  $0c
CR            .EQU  $0d
EOF           .EQU  $1a
LT            .EQU  $1d                   ; move one char left
SPC           .EQU  $20
BSL           .EQU  $5c

LINL40        .EQU  $f3ae                 ; system calls
LINL32        .EQU  $f3af
OLDMOD        .EQU  $fcb0
EXTBIO        .EQU  $ffca
_BDOS         .EQU  $0005
_STROUT       .EQU  $09                   ; string output

KNJMOD        .EQU  $f30f                 ; 4
H_PROM        .EQU  $f24f                 ; 3
DISKVE        .EQU  $f323                 ; 2
W_BOOT        .EQU  $0000                 ; 2
LINLEN        .EQU  $f3b0

ENASLT        .EQU  $0024
COMLIN        .EQU  $0080
RAMAD1        .EQU  $f342

MEGACON_HIGH  .EQU  $60
MEGACON_PORT  .EQU  MEGACON_HIGH*256
DOS2_BANK     .EQU  $00
ASMI_BANK     .EQU  $60
ASMI_SLOT     .EQU  %10001011             ; slot 3-2
ASMI_PORT     .EQU  $4000
ASMI_HIGH     .EQU  $5000
ACMD_RD_SID   .EQU  %10101011
ASMI_WR_STAT  .EQU  %00000001
ACMD_WRITE    .EQU  $02
ACMD_WR_ENA   .EQU  $06
ACMD_RD_STAT  .EQU  $05
ACMD_SECT_ERA .EQU  $d8

BUFFER        .EQU  $8000
BUFFER_SIZE   .EQU  $4000

ONE           .EQU  $01                   ;   1 byte  for char
HLN           .EQU  $10                   ;  16 bytes for hex line
CLU           .EQU  $0100                 ; 256 bytes for cluster

K1            .EQU  'K'                   ; editable copyleft: 1st three chars
K2            .EQU  'd'
K3            .EQU  'L'
; ----------------------------------------
startProg:
              jr    mainProg_start
; ----------------------------------------
typeMsg:
;             .DB   CR,SPC,SPC,SPC,SPC,LF ; 'jr mainProg_start' ctrl sequences mask
              .DB   CR,FN1+UP,FN2+UP,FN3+UP,"-"
              .DB   "PLD",FN4+UP,FN5+UP,FN6+UP,FN7+UP,FN8+UP
              .DB   " v",VER1,".",VER2," by "
clearCopyleft:
              .DB   K1,K2,K3," 2019-",YR1,YR2,YR3,YR4,CR
msgLF:
              .DB   LF,EOF
; ----------------------------------------
asmiInit:
              ld    h,MEGACON_HIGH        ; init connection with epcs
              ld    a,ASMI_SLOT
              call  ENASLT
              ld    hl,MEGACON_PORT
              ld    (hl),ASMI_BANK
              ret
asmiClose:
              ld    hl,MEGACON_PORT       ; close connection with epcs
              ld    (hl),DOS2_BANK
              ld    a,(RAMAD1)
              ld    h,MEGACON_HIGH
              jp    ENASLT
mainProg_start:
;             ld    hl,$3e00              ; kernel check (no longer needed)
;             ld    de,$0000
;             ld    bc,$01f0
;             xor   a
;             scf
;             ccf
;             rst   30
;             .DB   %10001011
;             .DW   $4010
;             ld    de,_msgerr1
;             jp    c,_ERR_MSG            ; kernel error
              call  _STDOS
              jp    nz,mainProg_end
              ld    hl,COMLIN+1           ; main
              ld    de,fileObject
              call  _STSTR
              jp    z,mainProg_end
              ld    de,fileObject
              ld    b,0
              call  _OPEN
              jp    nz,mainProg_end
              push  bc
; ----------------------------------------
chkMode:
              ld    de,$1100              ; check kanji mode
              call  EXTBIO
              cp    0
              jr    nz,mode80
              ld    a,(OLDMOD)            ; check mode
              cp    0
              jr    nz,mode32
              ld    a,(LINL40)
              cp    80                    ; check width 80
              jr    nz,mode40
mode80:
              ld    de,_msgwarn80
              jr    chkSID
mode40:
              cp    40                    ; check width 40
              jr    nz,mode80
              ld    de,_msgwarn40
              jr    chkSID
mode32:
              ld    a,(LINL32)            ; check width 32
              cp    32
              jr    nz,mode80
              ld    de,_msgwarn32
chkSID:
              push  de
              call  asmiInit
              ld    hl,ASMI_PORT
              ld    (hl),ACMD_RD_SID
              ld    (hl),a                ; 1st dummy byte
              ld    (hl),a                ; 2nd dummy byte
              ld    (hl),a                ; 3rd dummy byte
              ld    a,(hl)                ; 1st attempt
              ld    a,(hl)                ; 2nd attempt
              ld    a,(hl)                ; 3rd attempt
              ld    a,(hl)                ; 4th attempt
              ld    a,(hl)                ; 5th attempt
              ld    (chkSID),a            ; a = silicon id
              ld    a,(ASMI_HIGH)
              call  asmiClose
              pop   de
              ld    a,(chkSID)
              cp    %00010000             ; error if epcs1
              jr    z,errDevice
              cp    %00010010             ; error if epcs4
              jr    nz,printWarning
errDevice:
              ld    de,_msgerr0           ; device error
              jp    _ERR_MSG
printWarning:                             ; msg warning
              call  _PRINT
; ----------------------------------------
              ld    a,OCMID
              out   (SWIOADDR),a          ; set OCM ID
              ld    a,$3b
              out   (SCMDADDR),a          ; lock all toggles
              ld    a,$0a
              out   (SCMDADDR),a          ; set cpu to max speed
; ----------------------------------------
              pop   bc
              ld    (fileHandle),bc
              ld    sp,(_BDOS+1)
makeTable:
              ld    d,0
              ld    hl,reverseTable
              ld    l,d
makeTable_loop:
              ld    a,l
              ld    b,8
makeTable_bit:
              rlca
              rr    c
              djnz  makeTable_bit
              ld    (hl),c
              inc   l
              dec   d
              jr    nz,makeTable_loop
; ----------------------------------------
              ld    de,_msgepcs           ; epcs msg
              call  _PRINT
;             ld    de,_msgerase          ; erasing msg
              call  _PRINT
eraseAll:
              ld    c,32                  ; amount of sectors to be erase
              push  bc
eraseAll_loop:                            ; typ 22s with m25p64 device
              call  asmiInit
              ld    hl,ASMI_PORT
              ld    (hl),ACMD_WR_ENA
              ld    a,(ASMI_HIGH)
              xor   a
              pop   bc
              dec   c
              ld    (hl),ACMD_SECT_ERA
              ld    (hl),c
              ld    (hl),a
              ld    (hl),a
              ld    a,(ASMI_HIGH)
              push  bc
              ld    (hl),ACMD_RD_STAT
eraseAll_wait:
              ld    a,(hl)
              and   ASMI_WR_STAT
              jr    nz,eraseAll_wait
              ld    a,(ASMI_HIGH)
              call  asmiClose
              ld    c,$02
              ld    a,(_msgdata0)         ; 1st digit
              ld    e,a
              call  _BDOS
              ld    a,(_msgdata0+1)       ; 2nd digit
              ld    e,a
              call  _BDOS
              ld    e,LT                  ; left x2
              call  _BDOS
              call  _BDOS
              ld    hl,(_msgaddr0)        ; right shift by two chars
              ld    a,(hl)
              ld    (_msgdata0),a
              inc   hl
              ld    a,(hl)
              ld    (_msgdata0+1),a
              inc   hl
              ld    (_msgaddr0),hl
              xor   0
              jr    nz,eraseAll_loop
              pop   bc                    ; ok msg
; ----------------------------------------
              ld    de,_msgepcs           ; epcs msg
              call  _PRINT
              ld    de,_msgprog           ; programming msg
              call  _PRINT
              xor   a
              ld    h,a
              ld    l,a
mainProg_loop:                            ; typ 30s with m25p64 device
              push  hl
              push  af
              ld    de,BUFFER
              ld    hl,BUFFER_SIZE
              ld    bc,(fileHandle)
              call  _READ
              jp    nz,mainProg_end
              ld    a,h
              or    l
              jp    z,mainProg_end
              push  hl
              call  asmiInit
              pop   bc
              pop   af
              pop   hl
              push  hl
              push  af
              push  bc
              ld    de,BUFFER
; ----------------------------------------
; IN   a:hl   write address (l needs zero)
;        bc   write size (c needs zero)
;        de   source address
writePLD:
              exx
              ld    hl,ASMI_PORT
              ld    de,reverseTable
              exx
              ld    c,b
              ld    b,0
              ex    de,hl
              ld    e,d
              ld    d,a
writePLD_loop:
              push  de
              exx
              ld    (hl),ACMD_WR_ENA
              ld    a,(ASMI_HIGH)
              pop   bc
              ld    (hl),ACMD_WRITE
              ld    (hl),b
              ld    (hl),c
              ld    (hl),$00
              exx
writePLD_bytes:                           ; error detection is not performed
              ld    a,(hl)
              inc   hl
              exx
              ld    e,a
              ld    a,(de)
              ld    (hl),a
              exx
              djnz  writePLD_bytes
              ld    a,(ASMI_HIGH)
              push  hl
              push  de
              exx
              ld    (hl),ACMD_RD_STAT
writePLD_wait:
              ld    a,(hl)
              and   ASMI_WR_STAT
              jr    nz,writePLD_wait
              exx
              ld    a,(ASMI_HIGH)
              pop   de
              pop   hl
              inc   de
              dec   c
              jr    nz,writePLD_loop
              call  asmiClose
; ----------------------------------------
              ld    c,$02
              ld    a,(_msgdata)          ; 1st digit
              ld    e,a
              call  _BDOS
              ld    a,(_msgdata+1)        ; 2nd digit
              ld    e,a
              call  _BDOS
              ld    e,LT                  ; left x2
              call  _BDOS
              call  _BDOS
              ld    hl,(_msgaddr)         ; right shift by two chars
              ld    a,(hl)
              ld    (_msgdata),a
              inc   hl
              ld    a,(hl)
              ld    (_msgdata+1),a
              inc   hl
              ld    (_msgaddr),hl
              xor   0
              jr    nz,mainProg_count
              ld    e,LF
              call  _BDOS                 ; ok msg
              jr    mainProg_end
mainProg_count:
              pop   de
              pop   af
              pop   hl
              add   hl,de
              adc   a,0
              jp    mainProg_loop
mainProg_end:
              ld    bc,(fileHandle)
              call  _CLOSE
              call  _OSEND
; ----------------------------------------
              ld    a,$d5
              out   (SCMDADDR),a          ; set cpu to default speed
              ld    a,$3c
              out   (SCMDADDR),a          ; unlock all toggles
              xor   a
              out   (SWIOADDR),a          ; cancel Device ID
; ----------------------------------------
; BEGIN FILE LIBRARY (LITE VERSION)
; ----------------------------------------
; ENTRY       "_EXIT"
;             >,A=EXIT CODE
;             <,/
;             *,/
;             I,/
_EXIT:
              ld    b,a
              ld    a,(OSVER)
              dec   a
              jr    nz,_EXIT_H
              ld    de,_msgerr            ; error msg
_ERR_MSG:
              call  _PRINT
_EXIT_L:
              ld    c,$00
              call  _BDOS                 ; exit
_EXIT_H:
              ld    a,(_msgdata)
              xor   0
              jr    z,_DONE
              ld    de,_msghlp            ; help msg
              call  _PRINT
              jr    _EXIT_L
_DONE:
              ld    de,_msgend            ; end msg
              call  _PRINT
              ld    c,$62
              call  _BDOS                 ; exit with error code (b=0 no error)
; ----------------------------------------
; ENTRY       "UPPERB"
;             >,A=CHR CODE
;             <,A=CHANGED CHR CODE
;             *,AF
;             I,/
UPPERB:
              cp    'a'
              ret   c
              cp    'z'+1
              ret   nc
              add   a,'A'-'a'
              ret
; ----------------------------------------
; ENTRY       "_STDOS"
;             >,/
;             <,NZ;ERROR/Z;OK
;             *,AF,BC,DE,HL,AF',BC',DE',HL',IX,IY
;             I,?
_STDOS:                                   ; 34 bytes
              ld    c,$6f
              call  _BDOS
              or    a
              ret   nz
              inc   a
              cp    b
              jr    nc,_STDOS1
              ld    a,b
_STDOS1:
              ld    (OSVER),a
              ld    c,$19
              call  _BDOS
              inc   a
              ld    (CURDRV),a
              ld    hl,$0080
              ld    c,(hl)
              ld    b,h
              inc   hl
              add   hl,bc
              ld    (hl),b
              ld    de,WOKBUF
              ld    hl,KNJMOD
              ld    bc,4
              ldir
              ld    hl,H_PROM
              ld    c,3
              ldir
              ld    hl,DISKVE
              ld    c,2
              ldir
              ld    hl,W_BOOT+1
              ld    c,2
              ldir
              ld    a,(OSVER)
              dec   a
              ld    hl,_STDOS00
              ld    de,STDOUT
              call  z,_STSTR_L
              cp    a
              ret
_STDOS00:
              .DB   "CON",0
_STSTR_L:
              inc   hl
              ld    a,(hl)
              dec   hl
              cp    ':'
CURDRV_AT:
              xor   a
              jr    nz,_STSTR_L1
              ld    a,(hl)
              inc   hl
              inc   hl
              call  UPPERB
              sub   'A'
              and   $07
              inc   a
_STSTR_L1:
              ld    (de),a
              inc   de
              ld    bc,$0800+'.'
              call  _STSTR_L2
              ld    bc,$0300+'/'
              call  _STSTR_L2
              push  hl
              ld    bc,37-(1+8+3)-1
              ld    h,d
              ld    l,e
              inc   de
              ld    (hl),b
              ldir
              pop   hl
              dec   b
              ret
_STSTR_L2:
              ld    a,(hl)
              or    a
              jr    z,_STSTR_L3
              cp    $20
              jr    z,_STSTR_L3
              cp    $09
              jr    z,_STSTR_L3
              cp    '/'
              jr    z,_STSTR_L3
              inc   hl
              cp    '*'
              jr    z,_STSTR_L4
              cp    c
              jr    z,_STSTR_L5
              ld    (de),a
              inc   de
              djnz  _STSTR_L2
              jr    _STSTR_L7
_STSTR_L3:
              ld    a,8
              cp    b
              jr    nz,_STSTR_L5
              ld    b,8+3
_STSTR_L4:
              ld    a,'?'
              jr    z,_STSTR_L6
_STSTR_L5:
              ld    a,$20
_STSTR_L6:
              ld    (de),a
              inc   de
              djnz  _STSTR_L6
_STSTR_L7:
              ld    b,4
              push  hl
_STSTR_L8:
              ld    a,(hl)
              inc   hl
              or    a
              jr    z,_STSTR_L10
              cp    $20
              jr    z,_STSTR_L10
              cp    $09
              jr    z,_STSTR_L10
              cp    '/'
              jr    z,_STSTR_L10
              cp    c
              jr    z,_STSTR_L9
              djnz  _STSTR_L8
              pop   hl
              ret
_STSTR_L9:
              ex    (sp),hl
              pop   hl
              ret
_STSTR_L10:
              pop   hl
              ret
; ----------------------------------------
; ENTRY       "_STSTR"
;             >,HL=COMMAND LINE STRINGS,DE=BUF(67 bytes)
;             <,Z;ERROR/NZ;HL=HL+LEN(STRINGS)
;             *,AF,BC,DE
;             I,/
_STSTR:
              ld    a,(hl)
              or    a
              ret   z
              cp    '/'
              ret   z
              cp    $20
              jr    z,_STSTR1
              cp    $09
              jr    nz,_STSTR2
_STSTR1:
              inc   hl
              jr    _STSTR
_STSTR2:
              ld    a,(OSVER)
              dec   a
              jp    z,_STSTR_L
_STSTR_H:
              inc   hl
              ld    a,(hl)
              dec   hl
              cp    ':'
              jr    z,_STSTR_H0
              ld    a,(CURDRV)
              add   a,'A'-1
              ld    (de),a
              inc   de
              ld    a,':'
              ld    (de),a
              inc   de
_STSTR_H0:
              ld    b,2+1+64-1
_STSTR_H1:
              ld    a,(hl)
              ld    (de),a
              or    a
              jr    z,_STSTR_H3
              sub   $09
              jr    z,_STSTR_H3
              sub   $20-$09
              jr    z,_STSTR_H3
              sub   '/'-$20
              jr    z,_STSTR_H3
              inc   hl
              inc   de
              djnz  _STSTR_H1
_STSTR_H2:
              ld    a,(hl)
              or    a
              jr    z,_STSTR_H3
              sub   $09
              jr    z,_STSTR_H3
              sub   $20-$09
              jr    z,_STSTR_H3
              sub   '/'-$20
              jr    z,_STSTR_H3
              inc   hl
              jr    _STSTR_H2
_STSTR_H3:
              ld    (de),a
              dec   a
              ret
; ----------------------------------------
; ENTRY       "_OSEND"
;             >,/
;             <,/
;             *,AF,BC,DE,HL
;             I,/
_OSEND:
              ld    hl,WOKBUF
              ld    de,KNJMOD
              ld    bc,4
              ldir
              ld    de,H_PROM
              ld    c,3
              ldir
              ld    de,DISKVE
              ld    c,2
              ldir
              ld    de,W_BOOT+1
              ld    c,2
              ldir
              ret
; ----------------------------------------
; ENTRY       "_OPEN"
;             >,DE=FIB|FILE STRINGS(GIVEN BY _STSTR)|BUF(USED IN _xSRCH)
;             <,NZ;ERROR/Z;BC=EXTENDED FILE HANDLE
;             *,AF,BC,DE,HL,AF',BC',DE',HL',IX,IY
;             I,?
_OPEN:
              ld    a,(OSVER)
              dec   a
              jr    nz,_OPEN_H
_OPEN_L:
              push  de
              ld    c,$0f
              call  _BDOS
              or    a
              pop   bc
              ret   nz
_OPEN_L1:
              ld    hl,+14
              add   hl,bc
              ld    (hl),$01
              inc   hl
              ld    (hl),a
              ld    hl,+33
              add   hl,bc
              ld    (hl),a
              inc   hl
              ld    (hl),a
              inc   hl
              ld    (hl),a
              inc   hl
              ld    (hl),a
              ret
_OPEN_H:
              xor   a
              ld    c,$43
              call  _BDOS
              or    a
              ret
; ----------------------------------------
; ENTRY       "_SEEK"
;             >,DE:HL=FILE POINTER,A=OFFSET CODE,BC=EXTENDED FILE HANDLE
;             <,DE:HL=NEW FILE POINTER
;             *,AF,BC,DE,HL,AF',BC',DE',HL',IX,IY
;             I,?
_SEEK:
              ex    af,af'
OSVER_AT:
              xor   a
              dec   a
              jr    nz,_SEEK_H
_SEEK_L:
              push  bc
              pop   ix
              ex    af,af'
              or    a
              jr    z,_SEEK_L0
              dec   a
              jr    z,_SEEK_L1
_SEEK_L2:
              ld    a,(ix+16)
              add   a,l
              ld    l,a
              ld    a,(ix+17)
              adc   a,h
              ld    h,a
              ld    a,(ix+18)
              adc   a,e
              ld    e,a
              ld    a,(ix+19)
              adc   a,d
              ld    d,a
              jr    _SEEK_L0
_SEEK_L1:
              ld    a,(ix+33)
              add   a,l
              ld    l,a
              ld    a,(ix+34)
              adc   a,h
              ld    h,a
              ld    a,(ix+35)
              adc   a,e
              ld    e,a
              ld    a,(ix+36)
              adc   a,d
              ld    d,a
_SEEK_L0:
              ld    (ix+33),l
              ld    (ix+34),h
              ld    (ix+35),e
              ld    (ix+36),d
              ret
_SEEK_H:
              ex    af,af'
              ld    c,$4a
              jp    _BDOS
; ----------------------------------------
; ENTRY       "_READ"
;             >,BC=EXTENDED FILE HANDLE,DE=BUF,HL=SIZE
;             <,NZ;ERROR/Z;HL=SIZE
;             *,AF,BC,DE,HL,AF',BC',DE',HL',IX,IY
;             I,?
_READ:
              ld    a,(OSVER)
              dec   a
              jr    nz,_READ_H
_READ_L:
              push  bc
              push  hl
              ld    c,$1a
              call  _BDOS
              pop   hl
              pop   de
              ld    c,$27
              call  _BDOS
              or    a
              ret   z
              ld    a,l
              or    h
              jr    z,_READ_L1
              xor   a
              ret
_READ_L1:
              dec   a
              ret
_READ_H:
              ld    c,$48
              call  _BDOS
              or    a
              ret
; ----------------------------------------
; ENTRY       "_CLOSE"
;             >,BC=EXTENDED FILE HANDLE
;             <,NZ;ERROR/Z;OK
;             *,AF,BC,DE,HL,AF',BC',DE',HL',IX,IY
;             I,?
_CLOSE:
              ld    a,(OSVER)
              dec   a
              jr    nz,_CLOSE_H
_CLOSE_L:
              push  bc
              pop   de
              ld    c,$10
              call  _BDOS
              or    a
              ret
_CLOSE_H:
              ld    c,$45
              call  _BDOS
              or    a
              ret
; ----------------------------------------
; ENTRY       "_PRINT"
;             >,DE=STRING POINTER(USE CALL _PRINT)
;             <,/
;             *,BC,DE
;             I,/
_PRINT:
              ld    c,_STROUT
              jp    _BDOS
; ----------------------------------------
; END FILE LIBRARY (LITE VERSION)
; ----------------------------------------
_msgwarn32:
              .DB   "WARNING!! Do NOT turn off the",LF,CR
              .DB   "device until the Flash Process",LF,CR
              .DB   "is done.",LF,CR,'$'
_msgwarn40:
              .DB   "WARNING!! Do NOT turn off the device",LF,CR
              .DB   "until the Flash Process is done.",LF,CR,'$'
_msgwarn80:
              .DB   "WARNING!! Do NOT turn off the device "
              .DB   "until the Flash Process is done.",LF,CR,'$'
_msghlp:
              .DB   CLS,"Syntax: ",FN1,FN2,FN3,FN4,FN5,FN6,FN7,FN8
              .DB   " [path",BSL,"]filename.pld",LF,CR,LF
              .DB   "No parameter:    Shows this help",LF,CR,LF
              .DB   "A corrupted PLD file might cause",LF,CR
              .DB   "your system to FAIL!!",LF,CR,LF
              .DB   "Any damages caused by incorrect",LF,CR
              .DB   "firmware update operations",LF,CR
              .DB   "is at your own risk.",LF,CR,LF
              .DB   "Please be responsible.",LF,CR,CR,'$'
;_msgerr1:
;             .DB   LF,CR,"*** Unsupported kernel found",LF,CR,'$'
_msgerr0:
              .DB   LF,CR,"*** Unsupported device",LF,CR,'$'
_msgerr:
              .DB   LF,CR,"*** MSX-DOS 2 is required",LF
_msgend:
              .DB   CR,'$'
_msgepcs:
              .DB   LF,CR,"(EPCS16+) ",'$'
_msgerase:
              .DB   "Erasing device ... ",'$'
_msgprog:
              .DB   "Flashing device .. ",'$'
_msgaddr0:
              .DW   _msgdata0+2
_msgdata0:
              .DB   "  302928","27262524","23222120","19181716"
              .DB   "15141312","11109 8 ","7 6 5 4 ","3 2 1 OK"
              .DW   $0000                                       ; 32 sectors
_msgaddr:
              .DW   _msgdata+2
_msgdata:
              .DB   "        ","30303030","29292929","28282828" ; 32*8 kB
              .DB   "27272727","26262626","25252525","24242424" ; 32*8 kB
              .DB   "23232323","22222222","21212121","20202020" ; 32*8 kB
              .DB   "19191919","18181818","17171717","16161616" ; 32*8 kB
              .DB   "15151515","14141414","13131313","12121212" ; 32*8 kB
              .DB   "11111111","10101010","9 9 9 9 ","8 8 8 8 " ; 32*8 kB
              .DB   "7 7 7 7 ","6 6 6 6 ","5 5 5 5 ","4 4 4 4 " ; 32*8 kB
              .DB   "3 3 3 3 ","2 2 2 2 ","1 1 1 1 ","OKOKOKOK" ; 32*8 kB
;                                                               = 2048 kB
              .DW   $0000
workArea:
; ----------------------------------------
startFill:                                ; $FF fill the 1st hex line
;             .FILL ((((startFill-startProg)/HLN)+ONE)*HLN-(startFill-startProg))
endProg:                                  ; $FF fill the cluster
;             .FILL ((((endProg-startProg)/CLU)+ONE)*CLU-(endProg-startProg))-HLN
                                          ; [ FILENAME.COM ] at last hex line
              .DB   "[ ",FN1+UP,FN2+UP,FN3+UP,FN4+UP,FN5+UP,FN6+UP,FN7+UP,FN8+UP
              .DB   ".COM ]"

STDOUT        .EQU  _STSTR_H
WOKBUF        .EQU  _STDOS
FILTMP        .EQU  WOKBUF+4+3+2+2
OSVER         .EQU  OSVER_AT+1
CURDRV        .EQU  CURDRV_AT+1
fileObject    .EQU  workArea+0
fileHandle    .EQU  fileObject+67
reverseTable  .EQU  fileHandle+2+($0100-1)

              .END

;
; -----------------------------------------------------------------------------
; LICENSE-2.0 inherited from PLDLOAD v1.00 by NAKAYAMA Atsushi
; -----------------------------------------------------------------------------
;
; Apache License
; Version 2.0, January 2004
; http://www.apache.org/licenses/
;
; TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION
;
; 1. Definitions.
;
; "License" shall mean the terms and conditions for use, reproduction,
; and distribution as defined by Sections 1 through 9 of this document.
;
; "Licensor" shall mean the copyright owner or entity authorized by
; the copyright owner that is granting the License.
;
; "Legal Entity" shall mean the union of the acting entity and all
; other entities that control, are controlled by, or are under common
; control with that entity. For the purposes of this definition,
; "control" means (i) the power, direct or indirect, to cause the
; direction or management of such entity, whether by contract or
; otherwise, or (ii) ownership of fifty percent (50%) or more of the
; outstanding shares, or (iii) beneficial ownership of such entity.
;
; "You" (or "Your") shall mean an individual or Legal Entity
; exercising permissions granted by this License.
;
; "Source" form shall mean the preferred form for making modifications,
; including but not limited to software source code, documentation
; source, and configuration files.
;
; "Object" form shall mean any form resulting from mechanical
; transformation or translation of a Source form, including but
; not limited to compiled object code, generated documentation,
; and conversions to other media types.
;
; "Work" shall mean the work of authorship, whether in Source or
; Object form, made available under the License, as indicated by a
; copyright notice that is included in or attached to the work
; (an example is provided in the Appendix below).
;
; "Derivative Works" shall mean any work, whether in Source or Object
; form, that is based on (or derived from) the Work and for which the
; editorial revisions, annotations, elaborations, or other modifications
; represent, as a whole, an original work of authorship. For the purposes
; of this License, Derivative Works shall not include works that remain
; separable from, or merely link (or bind by name) to the interfaces of,
; the Work and Derivative Works thereof.
;
; "Contribution" shall mean any work of authorship, including
; the original version of the Work and any modifications or additions
; to that Work or Derivative Works thereof, that is intentionally
; submitted to Licensor for inclusion in the Work by the copyright owner
; or by an individual or Legal Entity authorized to submit on behalf of
; the copyright owner. For the purposes of this definition, "submitted"
; means any form of electronic, verbal, or written communication sent
; to the Licensor or its representatives, including but not limited to
; communication on electronic mailing lists, source code control systems,
; and issue tracking systems that are managed by, or on behalf of, the
; Licensor for the purpose of discussing and improving the Work, but
; excluding communication that is conspicuously marked or otherwise
; designated in writing by the copyright owner as "Not a Contribution."
;
; "Contributor" shall mean Licensor and any individual or Legal Entity
; on behalf of whom a Contribution has been received by Licensor and
; subsequently incorporated within the Work.
;
; 2. Grant of Copyright License. Subject to the terms and conditions of
; this License, each Contributor hereby grants to You a perpetual,
; worldwide, non-exclusive, no-charge, royalty-free, irrevocable
; copyright license to reproduce, prepare Derivative Works of,
; publicly display, publicly perform, sublicense, and distribute the
; Work and such Derivative Works in Source or Object form.
;
; 3. Grant of Patent License. Subject to the terms and conditions of
; this License, each Contributor hereby grants to You a perpetual,
; worldwide, non-exclusive, no-charge, royalty-free, irrevocable
; (except as stated in this section) patent license to make, have made,
; use, offer to sell, sell, import, and otherwise transfer the Work,
; where such license applies only to those patent claims licensable
; by such Contributor that are necessarily infringed by their
; Contribution(s) alone or by combination of their Contribution(s)
; with the Work to which such Contribution(s) was submitted. If You
; institute patent litigation against any entity (including a
; cross-claim or counterclaim in a lawsuit) alleging that the Work
; or a Contribution incorporated within the Work constitutes direct
; or contributory patent infringement, then any patent licenses
; granted to You under this License for that Work shall terminate
; as of the date such litigation is filed.
;
; 4. Redistribution. You may reproduce and distribute copies of the
; Work or Derivative Works thereof in any medium, with or without
; modifications, and in Source or Object form, provided that You
; meet the following conditions:
;
; (a) You must give any other recipients of the Work or
; Derivative Works a copy of this License; and
;
; (b) You must cause any modified files to carry prominent notices
; stating that You changed the files; and
;
; (c) You must retain, in the Source form of any Derivative Works
; that You distribute, all copyright, patent, trademark, and
; attribution notices from the Source form of the Work,
; excluding those notices that do not pertain to any part of
; the Derivative Works; and
;
; (d) If the Work includes a "NOTICE" text file as part of its
; distribution, then any Derivative Works that You distribute must
; include a readable copy of the attribution notices contained
; within such NOTICE file, excluding those notices that do not
; pertain to any part of the Derivative Works, in at least one
; of the following places: within a NOTICE text file distributed
; as part of the Derivative Works; within the Source form or
; documentation, if provided along with the Derivative Works; or,
; within a display generated by the Derivative Works, if and
; wherever such third-party notices normally appear. The contents
; of the NOTICE file are for informational purposes only and
; do not modify the License. You may add Your own attribution
; notices within Derivative Works that You distribute, alongside
; or as an addendum to the NOTICE text from the Work, provided
; that such additional attribution notices cannot be construed
; as modifying the License.
;
; You may add Your own copyright statement to Your modifications and
; may provide additional or different license terms and conditions
; for use, reproduction, or distribution of Your modifications, or
; for any such Derivative Works as a whole, provided Your use,
; reproduction, and distribution of the Work otherwise complies with
; the conditions stated in this License.
;
; 5. Submission of Contributions. Unless You explicitly state otherwise,
; any Contribution intentionally submitted for inclusion in the Work
; by You to the Licensor shall be under the terms and conditions of
; this License, without any additional terms or conditions.
; Notwithstanding the above, nothing herein shall supersede or modify
; the terms of any separate license agreement you may have executed
; with Licensor regarding such Contributions.
;
; 6. Trademarks. This License does not grant permission to use the trade
; names, trademarks, service marks, or product names of the Licensor,
; except as required for reasonable and customary use in describing the
; origin of the Work and reproducing the content of the NOTICE file.
;
; 7. Disclaimer of Warranty. Unless required by applicable law or
; agreed to in writing, Licensor provides the Work (and each
; Contributor provides its Contributions) on an "AS IS" BASIS,
; WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
; implied, including, without limitation, any warranties or conditions
; of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A
; PARTICULAR PURPOSE. You are solely responsible for determining the
; appropriateness of using or redistributing the Work and assume any
; risks associated with Your exercise of permissions under this License.
;
; 8. Limitation of Liability. In no event and under no legal theory,
; whether in tort (including negligence), contract, or otherwise,
; unless required by applicable law (such as deliberate and grossly
; negligent acts) or agreed to in writing, shall any Contributor be
; liable to You for damages, including any direct, indirect, special,
; incidental, or consequential damages of any character arising as a
; result of this License or out of the use or inability to use the
; Work (including but not limited to damages for loss of goodwill,
; work stoppage, computer failure or malfunction, or any and all
; other commercial damages or losses), even if such Contributor
; has been advised of the possibility of such damages.
;
; 9. Accepting Warranty or Additional Liability. While redistributing
; the Work or Derivative Works thereof, You may choose to offer,
; and charge a fee for, acceptance of support, warranty, indemnity,
; or other liability obligations and/or rights consistent with this
; License. However, in accepting such obligations, You may act only
; on Your own behalf and on Your sole responsibility, not on behalf
; of any other Contributor, and only if You agree to indemnify,
; defend, and hold each Contributor harmless for any liability
; incurred by, or claims asserted against, such Contributor by reason
; of your accepting any such warranty or additional liability.
;
; END OF TERMS AND CONDITIONS
;
; APPENDIX: How to apply the Apache License to your work.
;
; To apply the Apache License to your work, attach the following
; boilerplate notice, with the fields enclosed by brackets "[]"
; replaced with your own identifying information. (Don't include
; the brackets!)  The text should be enclosed in the appropriate
; comment syntax for the file format. We also recommend that a
; file or class name and description of purpose be included on the
; same "printed page" as the copyright notice for easier
; identification within third-party archives.
;
; Copyright [yyyy] [name of copyright owner]
;
; Licensed under the Apache License, Version 2.0 (the "License");
; you may not use this file except in compliance with the License.
; You may obtain a copy of the License at
;
; http://www.apache.org/licenses/LICENSE-2.0
;
; Unless required by applicable law or agreed to in writing, software
; distributed under the License is distributed on an "AS IS" BASIS,
; WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
; See the License for the specific language governing permissions and
; limitations under the License.
;
; -----------------------------------------------------------------------------
;
